@@ -38,10 +38,11 @@
 
 // each structure has a pointer as a unique identifier that allows type checking at run time
 
-const char *const liq_attr_magic = "liq_attr";
+static const char *const liq_attr_magic = "liq_attr";
 static const char *const liq_image_magic = "liq_image";
 static const char *const liq_result_magic = "liq_result";
 static const char *const liq_remapping_result_magic = "liq_remapping_result";
+static const char *const maniac = "she's a maniac, maniac on the floor";
 static const char *const liq_freed_magic = "free";
 
 #define CHECK_STRUCT_TYPE(attr, kind) {					\
@@ -65,7 +66,6 @@ static const char *const liq_freed_magic = "free";
     }
 #define CHECK_USER_POINTER(ptr) liq_crash_if_invalid_pointer_given(ptr)
 
-#if 0
 struct liq_attr
 {
     const char *magic_header;
@@ -84,7 +84,6 @@ struct liq_attr
     liq_log_flush_callback_function *log_flush_callback;
     void *log_flush_callback_user_info;
 };
-#endif /* 0*/
 
 struct liq_image
 {
@@ -856,6 +855,8 @@ liq_error liq_set_log_flush_callback(liq_attr *attr, liq_log_flush_callback_func
 
 static void *liq_aligned_malloc(size_t size)
 {
+    return malloc (size);
+    /*
     unsigned char *ptr = malloc(size + 16);
     if (!ptr) {
         return NULL;
@@ -866,14 +867,18 @@ static void *liq_aligned_malloc(size_t size)
     assert(0 == (((uintptr_t)ptr) & 15));
     ptr[-1] = offset ^ 0x59; // store how much pointer was shifted to get the original for free()
     return ptr;
+    */
 }
 
 static void liq_aligned_free(void *inptr)
 {
+    /*
     unsigned char *ptr = inptr;
     size_t offset = ptr[-1] ^ 0x59;
     assert(offset > 0 && offset <= 16);
     free(ptr - offset);
+    */
+    free (inptr);
 }
 
 liq_attr* liq_attr_create_with_allocator(void* (*custom_malloc)(size_t), void (*custom_free)(void*))
