@@ -14,14 +14,77 @@
 
 #include <stdlib.h>
 #include <stddef.h>
+#include <math.h>
+#include <assert.h>
+#include <stdbool.h>
 
-#include "libimagequant.h"
 #include "pam.h"
 #include "mediancut.h"
 
 #define index_of_channel(ch) (offsetof(f_pixel,ch)/sizeof(float))
 
-static f_pixel averagepixels(unsigned int clrs, const hist_item achv[], float min_opaque_val, const f_pixel center);
+static f_pixel averagepixels(unsigned int clrs, const hist_item achv[], const float min_opaque_val, const f_pixel center)
+{
+    double r = 0, g = 0, b = 0, a = 0, new_a=0, sum = 0;
+    float maxa = 0;
+    int i;
+    // first find final opacity in order to blend colors at that opacity
+    for (i = 0; i < clrs; ++i) {
+        const f_pixel px = achv[i].acolor;
+        new_a += px.a * achv[i].adjusted_weight;
+        sum += achv[i].adjusted_weight;
+
+        /* find if there are opaque colors, in case we're supposed to preserve opacity exactly (ie_bug) */
+        if (px.a > maxa) maxa = px.a;
+    }
+
+    if (sum) new_a /= sum;
+
+    /** if there was at least one completely opaque color, "round" final color to opaque */
+    if (new_a >= min_opaque_val && maxa >= (255.0/256.0)) new_a = 1;
+
+    sum=0;
+    // reverse iteration for cache locality with previous loop
+    for(i = clrs-1; i >= 0; i--) {
+        double tmp, weight = 1.0f;
+        f_pixel px = achv[i].acolor;
+
+        /* give more weight to colors that are further away from average
+         this is intended to prevent desaturation of images and fading of whites
+         */
+        tmp = (center.r - px.r);
+        weight += tmp*tmp;
+        tmp = (center.g - px.g);
+        weight += tmp*tmp;
+        tmp = (center.b - px.b);
+        weight += tmp*tmp;
+
+        weight *= achv[i].adjusted_weight;
+        sum += weight;
+
+        if (px.a) {
+            px.r /= px.a;
+            px.g /= px.a;
+            px.b /= px.a;
+        }
+
+        r += px.r * new_a * weight;
+        g += px.g * new_a * weight;
+        b += px.b * new_a * weight;
+        a += new_a * weight;
+    }
+
+    if (sum) {
+    a /= sum;
+    r /= sum;
+    g /= sum;
+    b /= sum;
+    }
+
+    assert(!isnan(r) && !isnan(g) && !isnan(b) && !isnan(a));
+
+    return (f_pixel){.r=r, .g=g, .b=b, .a=a};
+}
 
 struct box {
     f_pixel color;
@@ -31,7 +94,6 @@ struct box {
     unsigned int colors;
 };
 
-inline static double variance_diff(double val, const double good_enough) ALWAYS_INLINE;
 inline static double variance_diff(double val, const double good_enough)
 {
     val *= val;
@@ -44,8 +106,8 @@ static f_pixel box_variance(const hist_item achv[], const struct box *box)
 {
     f_pixel mean = box->color;
     double variancea=0, variancer=0, varianceg=0, varianceb=0;
-
-    for(unsigned int i = 0; i < box->colors; ++i) {
+    unsigned int i;
+    for(i = 0; i < box->colors; ++i) {
         f_pixel px = achv[box->ind + i].acolor;
         double weight = achv[box->ind + i].adjusted_weight;
         variancea += variance_diff(mean.a - px.a, 2.0/256.0)*weight;
@@ -66,8 +128,8 @@ static double box_max_error(const hist_item achv[], const struct box *box)
 {
     f_pixel mean = box->color;
     double max_error = 0;
-
-    for(unsigned int i = 0; i < box->colors; ++i) {
+    unsigned int i;
+    for(i = 0; i < box->colors; ++i) {
         const double diff = colordifference(mean, achv[box->ind + i].acolor);
         if (diff > max_error) {
             max_error = diff;
@@ -76,7 +138,13 @@ static double box_max_error(const hist_item achv[], const struct box *box)
     return max_error;
 }
 
-inline static double color_weight(f_pixel median, hist_item h) ALWAYS_INLINE;
+inline static double color_weight(f_pixel median, hist_item h)
+{
+    float diff = colordifference(median, h.acolor);
+    // if color is "good enough", don't split further
+    if (diff < 2.f/256.f/256.f) diff /= 2.f;
+    return sqrt(diff) * (sqrt(1.0+h.adjusted_weight)-1.0);
+}
 
 static inline void hist_item_swap(hist_item *l, hist_item *r)
 {
@@ -87,7 +155,6 @@ static inline void hist_item_swap(hist_item *l, hist_item *r)
     }
 }
 
-inline static unsigned int qsort_pivot(const hist_item *const base, const unsigned int len) ALWAYS_INLINE;
 inline static unsigned int qsort_pivot(const hist_item *const base, const unsigned int len)
 {
     if (len < 32) return len/2;
@@ -98,7 +165,6 @@ inline static unsigned int qsort_pivot(const hist_item *const base, const unsign
                    : ((b > c) ? bidx : ((a < c) ? aidx : cidx ));
 }
 
-inline static unsigned int qsort_partition(hist_item *const base, const unsigned int len) ALWAYS_INLINE;
 inline static unsigned int qsort_partition(hist_item *const base, const unsigned int len)
 {
     unsigned int l = 1, r = len;
@@ -122,6 +188,7 @@ inline static unsigned int qsort_partition(hist_item *const base, const unsigned
 }
 
 /** quick select algorithm */
+
 static void hist_item_sort_range(hist_item *base, unsigned int len, unsigned int sort_start)
 {
     for(;;) {
@@ -170,50 +237,6 @@ static hist_item *hist_item_sort_halfvar(hist_item *base, unsigned int len, doub
     } while(1);
 }
 
-static f_pixel get_median(const struct box *b, hist_item achv[]);
-
-typedef struct {
-    unsigned int chan; float variance;
-} channelvariance;
-
-static int comparevariance(const void *ch1, const void *ch2)
-{
-    return ((const channelvariance*)ch1)->variance > ((const channelvariance*)ch2)->variance ? -1 :
-          (((const channelvariance*)ch1)->variance < ((const channelvariance*)ch2)->variance ? 1 : 0);
-}
-
-/** Finds which channels need to be sorted first and preproceses achv for fast sort */
-static double prepare_sort(struct box *b, hist_item achv[])
-{
-    /*
-     ** Sort dimensions by their variance, and then sort colors first by dimension with highest variance
-     */
-    channelvariance channels[4] = {
-        {index_of_channel(r), b->variance.r},
-        {index_of_channel(g), b->variance.g},
-        {index_of_channel(b), b->variance.b},
-        {index_of_channel(a), b->variance.a},
-    };
-
-    qsort(channels, 4, sizeof(channels[0]), comparevariance);
-
-    for(unsigned int i=0; i < b->colors; i++) {
-        const float *chans = (const float *)&achv[b->ind + i].acolor;
-        // Only the first channel really matters. When trying median cut many times
-        // with different histogram weights, I don't want sort randomness to influence outcome.
-        achv[b->ind + i].sort_value = ((unsigned int)(chans[channels[0].chan]*65535.0)<<16) |
-                                       (unsigned int)((chans[channels[2].chan] + chans[channels[1].chan]/2.0 + chans[channels[3].chan]/4.0)*65535.0);
-    }
-
-    const f_pixel median = get_median(b, achv);
-
-    // box will be split to make color_weight of each side even
-    const unsigned int ind = b->ind, end = ind+b->colors;
-    double totalvar = 0;
-    for(unsigned int j=ind; j < end; j++) totalvar += (achv[j].color_weight = color_weight(median, achv[j]));
-    return totalvar / 2.0;
-}
-
 /** finds median in unsorted set by sorting only minimum required */
 static f_pixel get_median(const struct box *b, hist_item achv[])
 {
@@ -229,13 +252,61 @@ static f_pixel get_median(const struct box *b, hist_item achv[])
     return averagepixels(2, &achv[b->ind + median_start], 1.0, (f_pixel){0.5,0.5,0.5,0.5});
 }
 
+typedef struct {
+    unsigned int chan;
+    float variance;
+}
+channelvariance;
+
+static int comparevariance(const void *ch1, const void *ch2)
+{
+    return ((const channelvariance*)ch1)->variance > ((const channelvariance*)ch2)->variance ? -1 :
+          (((const channelvariance*)ch1)->variance < ((const channelvariance*)ch2)->variance ? 1 : 0);
+}
+
+/** Finds which channels need to be sorted first and preproceses achv for fast sort */
+static double prepare_sort(struct box *b, hist_item achv[])
+{
+    unsigned int i;
+    unsigned int j;
+    /*
+     ** Sort dimensions by their variance, and then sort colors first by dimension with highest variance
+     */
+    channelvariance channels[4] = {
+        {index_of_channel(r), b->variance.r},
+        {index_of_channel(g), b->variance.g},
+        {index_of_channel(b), b->variance.b},
+        {index_of_channel(a), b->variance.a},
+    };
+
+    qsort(channels, 4, sizeof(channels[0]), comparevariance);
+
+    for (i=0; i < b->colors; i++) {
+        const float *chans = (const float *)&achv[b->ind + i].acolor;
+        // Only the first channel really matters. When trying median cut many times
+        // with different histogram weights, I don't want sort randomness to influence outcome.
+        achv[b->ind + i].sort_value = ((unsigned int)(chans[channels[0].chan]*65535.0)<<16) |
+                                       (unsigned int)((chans[channels[2].chan] + chans[channels[1].chan]/2.0 + chans[channels[3].chan]/4.0)*65535.0);
+    }
+
+    const f_pixel median = get_median(b, achv);
+
+    // box will be split to make color_weight of each side even
+    const unsigned int ind = b->ind, end = ind+b->colors;
+    double totalvar = 0;
+    for (j=ind; j < end; j++) totalvar += (achv[j].color_weight = color_weight(median, achv[j]));
+    return totalvar / 2.0;
+}
+
 /*
  ** Find the best splittable box. -1 if no boxes are splittable.
  */
 static int best_splittable_box(struct box* bv, unsigned int boxes, const double max_mse)
 {
-    int bi=-1; double maxsum=0;
-    for(unsigned int i=0; i < boxes; i++) {
+    unsigned int i;
+    int bi=-1;
+    double maxsum=0;
+    for (i=0; i < boxes; i++) {
         if (bv[i].colors < 2) continue;
 
         // looks only at max variance, because it's only going to split by it
@@ -254,23 +325,48 @@ static int best_splittable_box(struct box* bv, unsigned int boxes, const double
     return bi;
 }
 
-inline static double color_weight(f_pixel median, hist_item h)
+static void set_colormap_from_boxes(colormap *map, struct box* bv, unsigned int boxes, hist_item *achv)
 {
-    float diff = colordifference(median, h.acolor);
-    // if color is "good enough", don't split further
-    if (diff < 2.f/256.f/256.f) diff /= 2.f;
-    return sqrt(diff) * (sqrt(1.0+h.adjusted_weight)-1.0);
+    unsigned int bi;
+    /*
+     ** Ok, we've got enough boxes.  Now choose a representative color for
+     ** each box.  There are a number of possible ways to make this choice.
+     ** One would be to choose the center of the box; this ignores any structure
+     ** within the boxes.  Another method would be to average all the colors in
+     ** the box - this is the method specified in Heckbert's paper.
+     */
+
+    for (bi = 0; bi < boxes; ++bi) {
+	unsigned int i;
+        map->palette[bi].acolor = bv[bi].color;
+
+        /* store total color popularity (perceptual_weight is approximation of it) */
+        map->palette[bi].popularity = 0;
+        for (i=bv[bi].ind; i < bv[bi].ind+bv[bi].colors; i++) {
+            map->palette[bi].popularity += achv[i].perceptual_weight;
+        }
+    }
 }
 
-static void set_colormap_from_boxes(colormap *map, struct box* bv, unsigned int boxes, hist_item *achv);
-static void adjust_histogram(hist_item *achv, const colormap *map, const struct box* bv, unsigned int boxes);
+/* increase histogram popularity by difference from the final color (this is used as part of feedback loop) */
+static void adjust_histogram(hist_item *achv, const colormap *map, const struct box* bv, unsigned int boxes)
+{
+    unsigned int bi;
+    for (bi = 0; bi < boxes; ++bi) {
+	unsigned int i;
+        for (i=bv[bi].ind; i < bv[bi].ind+bv[bi].colors; i++) {
+            achv[i].adjusted_weight *= sqrt(1.0 +colordifference(map->palette[bi].acolor, achv[i].acolor)/4.0);
+            achv[i].likely_colormap_index = bi;
+        }
+    }
+}
 
-double box_error(const struct box *box, const hist_item achv[])
+static double box_error (const struct box *box, const hist_item achv[])
 {
     f_pixel avg = box->color;
-
+    unsigned int i;
     double total_error=0;
-    for (unsigned int i = 0; i < box->colors; ++i) {
+    for (i = 0; i < box->colors; ++i) {
         total_error += colordifference(avg, achv[box->ind + i].acolor) * achv[box->ind + i].perceptual_weight;
     }
 
@@ -282,8 +378,9 @@ static bool total_box_error_below_target(double target_mse, struct box bv[], uns
 {
     target_mse *= hist->total_perceptual_weight;
     double total_error=0;
+    unsigned int i;
 
-    for(unsigned int i=0; i < boxes; i++) {
+    for (i=0; i < boxes; i++) {
         // error is (re)calculated lazily
         if (bv[i].total_error >= 0) {
             total_error += bv[i].total_error;
@@ -291,7 +388,7 @@ static bool total_box_error_below_target(double target_mse, struct box bv[], uns
         if (total_error > target_mse) return false;
     }
 
-    for(unsigned int i=0; i < boxes; i++) {
+    for (i=0; i < boxes; i++) {
         if (bv[i].total_error < 0) {
             bv[i].total_error = box_error(&bv[i], hist->achv);
             total_error += bv[i].total_error;
@@ -307,10 +404,11 @@ static bool total_box_error_below_target(double target_mse, struct box bv[], uns
  ** on Paul Heckbert's paper, "Color Image Quantization for Frame Buffer
  ** Display," SIGGRAPH 1982 Proceedings, page 297.
  */
-LIQ_PRIVATE colormap *mediancut(histogram *hist, const float min_opaque_val, unsigned int newcolors, const double target_mse, const double max_mse, void* (*malloc)(size_t), void (*free)(void*))
+colormap *mediancut(histogram *hist, const float min_opaque_val, unsigned int newcolors, const double target_mse, const double max_mse, void* (*malloc)(size_t), void (*free)(void*))
 {
     hist_item *achv = hist->achv;
     struct box bv[newcolors];
+    unsigned int i;
 
     /*
      ** Set up the initial box.
@@ -322,7 +420,9 @@ LIQ_PRIVATE colormap *mediancut(histogram *hist, const float min_opaque_val, uns
     bv[0].max_error = box_max_error(achv, &bv[0]);
     bv[0].sum = 0;
     bv[0].total_error = -1;
-    for(unsigned int i=0; i < bv[0].colors; i++) bv[0].sum += achv[i].adjusted_weight;
+    for (i=0; i < bv[0].colors; i++) {
+	bv[0].sum += achv[i].adjusted_weight;
+    }
 
     unsigned int boxes = 1;
 
@@ -374,7 +474,7 @@ LIQ_PRIVATE colormap *mediancut(histogram *hist, const float min_opaque_val, uns
          */
         double sm = bv[bi].sum;
         double lowersum = 0;
-        for(unsigned int i=0; i < break_at; i++) lowersum += achv[indx + i].adjusted_weight;
+        for (i=0; i < break_at; i++) lowersum += achv[indx + i].adjusted_weight;
 
         const f_pixel previous_center = bv[bi].color;
         bv[bi].colors = break_at;
@@ -407,97 +507,4 @@ LIQ_PRIVATE colormap *mediancut(histogram *hist, const float min_opaque_val, uns
     return map;
 }
 
-static void set_colormap_from_boxes(colormap *map, struct box* bv, unsigned int boxes, hist_item *achv)
-{
-    /*
-     ** Ok, we've got enough boxes.  Now choose a representative color for
-     ** each box.  There are a number of possible ways to make this choice.
-     ** One would be to choose the center of the box; this ignores any structure
-     ** within the boxes.  Another method would be to average all the colors in
-     ** the box - this is the method specified in Heckbert's paper.
-     */
 
-    for(unsigned int bi = 0; bi < boxes; ++bi) {
-        map->palette[bi].acolor = bv[bi].color;
-
-        /* store total color popularity (perceptual_weight is approximation of it) */
-        map->palette[bi].popularity = 0;
-        for(unsigned int i=bv[bi].ind; i < bv[bi].ind+bv[bi].colors; i++) {
-            map->palette[bi].popularity += achv[i].perceptual_weight;
-        }
-    }
-}
-
-/* increase histogram popularity by difference from the final color (this is used as part of feedback loop) */
-static void adjust_histogram(hist_item *achv, const colormap *map, const struct box* bv, unsigned int boxes)
-{
-    for(unsigned int bi = 0; bi < boxes; ++bi) {
-        for(unsigned int i=bv[bi].ind; i < bv[bi].ind+bv[bi].colors; i++) {
-            achv[i].adjusted_weight *= sqrt(1.0 +colordifference(map->palette[bi].acolor, achv[i].acolor)/4.0);
-            achv[i].likely_colormap_index = bi;
-        }
-    }
-}
-
-static f_pixel averagepixels(unsigned int clrs, const hist_item achv[], const float min_opaque_val, const f_pixel center)
-{
-    double r = 0, g = 0, b = 0, a = 0, new_a=0, sum = 0;
-    float maxa = 0;
-
-    // first find final opacity in order to blend colors at that opacity
-    for(unsigned int i = 0; i < clrs; ++i) {
-        const f_pixel px = achv[i].acolor;
-        new_a += px.a * achv[i].adjusted_weight;
-        sum += achv[i].adjusted_weight;
-
-        /* find if there are opaque colors, in case we're supposed to preserve opacity exactly (ie_bug) */
-        if (px.a > maxa) maxa = px.a;
-    }
-
-    if (sum) new_a /= sum;
-
-    /** if there was at least one completely opaque color, "round" final color to opaque */
-    if (new_a >= min_opaque_val && maxa >= (255.0/256.0)) new_a = 1;
-
-    sum=0;
-    // reverse iteration for cache locality with previous loop
-    for(int i = clrs-1; i >= 0; i--) {
-        double tmp, weight = 1.0f;
-        f_pixel px = achv[i].acolor;
-
-        /* give more weight to colors that are further away from average
-         this is intended to prevent desaturation of images and fading of whites
-         */
-        tmp = (center.r - px.r);
-        weight += tmp*tmp;
-        tmp = (center.g - px.g);
-        weight += tmp*tmp;
-        tmp = (center.b - px.b);
-        weight += tmp*tmp;
-
-        weight *= achv[i].adjusted_weight;
-        sum += weight;
-
-        if (px.a) {
-            px.r /= px.a;
-            px.g /= px.a;
-            px.b /= px.a;
-        }
-
-        r += px.r * new_a * weight;
-        g += px.g * new_a * weight;
-        b += px.b * new_a * weight;
-        a += new_a * weight;
-    }
-
-    if (sum) {
-    a /= sum;
-    r /= sum;
-    g /= sum;
-    b /= sum;
-    }
-
-    assert(!isnan(r) && !isnan(g) && !isnan(b) && !isnan(a));
-
-    return (f_pixel){.r=r, .g=g, .b=b, .a=a};
-}
